@@ -117,37 +117,44 @@ def cb_params(trial):
 models = {
 
     # Gradient Boosts
-    'XGBR': lambda trial: XGBRegressor(**xgboost_params(trial)),
-    'XGBC': lambda trial: XGBClassifier(**xgboost_params(trial)),
-    'LGBR': lambda trial: LGBMRegressor(**lgb_params(trial)),
-    'LGBC': lambda trial: LGBMClassifier(**lgb_params(trial)),
+    'XGBR': (XGBRegressor, xgboost_params),
+    'XGBC': (XGBClassifier, xgboost_params),
+    'LGBR': (LGBMRegressor, lgb_params),
+    'LGBC': (LGBMClassifier, lgb_params),
 
     # Trees
-    'RFR': lambda trial: RandomForestRegressor(**rf_params(trial)),
-    'RFC': lambda trial: RandomForestClassifier(**rf_params(trial)),
-    'DTR': lambda trial: DecisionTreeRegressor(**dt_params(trial)),
-    'DTC': lambda trial: DecisionTreeClassifier(**dt_params(trial)),
-    'ETR': lambda trial: ExtraTreeRegressor(**et_params(trial)),
-    'ETC': lambda trial: ExtraTreeClassifier(**et_params(trial)),
+    'RFR': (RandomForestRegressor, rf_params),
+    'RFC': (RandomForestClassifier, rf_params),
+    'DTR': (DecisionTreeRegressor, dt_params),
+    'DTC': (DecisionTreeClassifier, dt_params),
+    'ETR': (ExtraTreeRegressor, et_params),
+    'ETC': (ExtraTreeClassifier, et_params),
 
     # Ensemble
-    'BC': lambda trial: BaggingClassifier(**bagg_params(trial)),
-    'BR': lambda trial: BaggingRegressor(**bagg_params(trial)),
-    'ADAR': lambda trial: AdaBoostRegressor(**ada_params(trial)),
-    'ADAC': lambda trial: AdaBoostClassifier(**ada_params(trial)),
-    'CBR': lambda trial: CatBoostRegressor(**cb_params(trial)),
-    'CBC': lambda trial: CatBoostClassifier(**cb_params(trial)),
+    'BC': (BaggingClassifier, bagg_params),
+    'BR': (BaggingRegressor, bagg_params),
+    'ADAR': (AdaBoostRegressor, ada_params),
+    'ADAC': (AdaBoostClassifier, ada_params),
+    'CBR': (CatBoostRegressor, cb_params),
+    'CBC': (CatBoostClassifier, cb_params),
 
     # KNeighbors
-    'KNC': lambda trial: KNeighborsClassifier(**kn_params(trial)),
-    'KNR': lambda trial: KNeighborsRegressor(**kn_params(trial)),
+    'KNC': (KNeighborsClassifier, kn_params),
+    'KNR': (KNeighborsRegressor, kn_params),
 
     # SVM
-    'SVR': lambda trial: SVR(**svm_params(trial)),
-    'SVC': lambda trial: SVC(**svm_params(trial)),
+    'SVR': (SVR, svm_params),
+    'SVC': (SVC, svm_params),
 }
 
 
+def get_model(model_name, trial, custom_params_fn=None):
+    model, params_fn = models[model_name]
+    params = params_fn(trial)
+    custom_params = custom_params_fn(trial) if custom_params_fn is not None else {}
+    custom_params.update(params)
+    return model(custom_params)
+
 class EarlyStoppingError(Exception):
     pass
 
@@ -174,10 +181,11 @@ class EarlyStopping:
 
 def optimize(model_name, scorer, y_column, trials=30, sampler=TPESampler(seed=666),
              direction='maximize', patience=100, threshold=1e-3, feature_selection_support=None,
-             feature_selection_cols=None, ds=None, path=None, sample_size=None, stratify=False):
+             feature_selection_cols=None, ds=None, path=None, sample_size=None, stratify=False,
+             custom_params_fn=None):
 
     if sample_size is not None:
-        ds, _ = train_test_split(ds, train_size=sample_size, stratify=ds[y_label] if stratify else None)
+        ds, _ = train_test_split(ds, train_size=sample_size, stratify=ds[y_column] if stratify else None)
 
     X_ds, y_ds = read_dataset(ds, path, y_column, feature_selection_support, feature_selection_cols)
     X_train, X_val, y_train, y_val = train_test_split(X_ds, y_ds)
@@ -186,7 +194,7 @@ def optimize(model_name, scorer, y_column, trials=30, sampler=TPESampler(seed=66
     stopping = EarlyStopping(direction, patience, threshold)
 
     def objective(trial):
-        model = models[model_name](trial)
+        model = get_model(model_name, trial, custom_params_fn)
         model.fit(X_train, y_train)
         preds = model.predict(X_val)
         error = get_score_fn(scorer)(y_val, preds)
